import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;

/**
   A class that demonstrates use of Graphics translation.
   This program draws an image whose top left corner is
   located at the position where the user clicks the mouse
   button.
*/
class GraphicsModesDemo extends JPanel
{
  /**
     No-arg constructor.  Creates the listener for the mouse
     clicks, and initially loads the image.  
  */
  public GraphicsModesDemo()
  {
    String path = "../images/snowy-peaks.gif";
    image = Toolkit.getDefaultToolkit().getImage(path);

    try
      {
	MediaTracker tracker = new MediaTracker(this);
	tracker.addImage(image, 0);
	tracker.waitForID(0);
      }
    catch (InterruptedException e)
      {

      }
    createListeners();
  }


  /**
     Creates the mouse listener that handles the user mouse
     clicks.  Upon a user click, the listener determines the
     location of the click and translates the graphics
     origin to that point.
  */
  private void createListeners()
  {
    MouseListener mL = new MouseHandler();
    MouseMotionListener mmL = new MouseMotionHandler();

    addMouseListener(mL);
    addMouseMotionListener(mmL);
  }


  public Dimension getPreferredSize()
  {
    return new Dimension(400, 300);
  }


  /**
     Paints the image.  This call displays the image the
     first time the applet is started.  Also, if the applet
     window is hidden and then raised, this method will
     redraw the applet at the location it was last drawn in
     response to last user mouse click.

     @param g the graphics created by the AWT and passed in.  
  */
  public void paint(Graphics g)
  {
    g.clearRect(0, 0, getWidth(), getHeight());
    g.drawImage(image, origin.x, origin.y, this);
  }


  public static void main(String [] args)
  {
    GraphicsModesDemo app = new GraphicsModesDemo();
    JFrame f = new JFrame("Graphics Modes Demo");
    WindowListener wl = new WindowAdapter()
      {
	public void windowClosing(WindowEvent e)
	  {
	    e.getWindow().dispose();
	    System.exit(0);
	  }
      };
    f.addWindowListener(wl);
    f.getContentPane().add(app);
    f.pack();
    f.setVisible(true);
  }

  // The image to display.
  //
  Image image;

  // Initializes a new Point which specifies the location of
  // the top left corner where the image should first be
  // drawn.  
  //
  Point origin = new Point(30, 20);


  // Indicates if the user has dragged the mouse yet.  If
  // so, the mouse motion listener needs to erase the
  // previously drawn rectangle in XOR mode, which will
  // revert the colors to their original state.
  //
  boolean firstDrag = true;


  // The point where the user pressed the mouse.  This marks
  // the top left corner of the rectangle drawn by the
  // MouseMotionHandler.
  //
  Point topLeft;


  // The bottom right corner of the rectangle drawn by the
  // the last drag event.
  Point lastDrag; 


  // The XOR mode alternation color used.
  //
  Color xorColor = Color.white;


  /**
     A class to handle the mouse events.  A mouse click
     redraws the image so that its top left corner is at the
     position of the click.  A mouse drag draws a rectangle
     around the area whose upper left corner is at the point
     where the mouse was pressed and whose lower right
     corner is at the point where the mouse was released.
  */
  public class MouseHandler extends MouseAdapter
  {
    /**
       Draws the image with its upper left corner at the
       point where the mouse was clicked.
    */
    public void mouseClicked(MouseEvent e)
    {
      // Determine where the user clicked the mouse.
      Point click = e.getPoint();

      Point lastPosition = origin;

      // Save the click location.  The paint method uses
      // this location to redraw the image in the last
      // location specified by a mouse click in response to
      // the window being hidden and then raised.
      //
      origin = click;

      // Get a new graphics; we don't have one yet.
      Graphics g = getGraphics();

      g.drawImage(image, lastPosition.x, lastPosition.y, 
		  GraphicsModesDemo.this);

      // Clear the whole applet so we don't have residue
      // from previous images drawn in other positions.
      g.clearRect(0, 0, getWidth(), getHeight());

      // Translate the origin of the Graphics to the point
      // clicked by the user in the applet's window.
      g.translate(click.x, click.y);

      // The image will now be drawn from the new origin.
      // Notice the new origin is referred to as the point
      // (0, 0).
      //
      g.drawImage(image, 0, 0, GraphicsModesDemo.this);

      // Dispose of the graphics that we created.
      //
      g.dispose();
    }


    /**
       Stores the point at which the mouse was pressed.
       This is used by the mouseDragged() method of the
       MouseMotionHandler to know the top left corner of the
       rectangle is draws.
    */
    public void mousePressed(MouseEvent e)
    {
      Graphics g = getGraphics();
      g.setXORMode(xorColor);

      // Erase the last rectangle from the previous drag if
      // there was one.
      //

      // Set up the data for this drag.
      //
      topLeft = e.getPoint();
      firstDrag = true;

      // Dispose of the graphics created here.
      g.dispose();
    }

    /**
       Erases the last rectangle drawn when the user releases
       the mouse.
     */
    public void mouseReleased(MouseEvent e)
    {
      Graphics g = getGraphics();

      g.setXORMode(xorColor);
      g.drawRect(topLeft.x, topLeft.y,
		 lastDrag.x - topLeft.x, 
		 lastDrag.y - topLeft.y);
      g.drawRect(topLeft.x + 1, topLeft.y + 1,
		 lastDrag.x + 1 - topLeft.x - 1, 
		 lastDrag.y + 1 - topLeft.y - 1);
      g.dispose();
    }

  }


  /**
     A class that defines listeners that draw a rectangle in
     XOR mode around a region whose top left and bottom
     right corners are given by the point where the user
     presses the mouse and the point where it is released,
     respectively.
   */
  public class MouseMotionHandler extends MouseMotionAdapter
  {
    public void mouseDragged(MouseEvent e)
    {
      Graphics g = getGraphics();
      g.setXORMode(xorColor);

      if (firstDrag)
	{
	  // There's no rectangle to erase.
	  firstDrag = false;
	}
      else
	{
	  // Must erase a previouly drawn rectangle.  In
	  // graphics XOR mode, drawing over a rectangle
	  // previously drawn in XOR mode reverts those bits
	  // to their original color.
	  //
 	  g.drawRect(topLeft.x, topLeft.y,
		     lastDrag.x - topLeft.x, 
		     lastDrag.y - topLeft.y);
	  g.drawRect(topLeft.x + 1, topLeft.y + 1, 
		     lastDrag.x + 1 - topLeft.x - 1, 
		     lastDrag.y + 1 - topLeft.y - 1);
	}

      // Finally, draw the new rectangle.
      g.drawRect(topLeft.x, topLeft.y, 
		 e.getPoint().x - topLeft.x, 
		 e.getPoint().y - topLeft.y);
      g.drawRect(topLeft.x + 1, topLeft.y + 1, 
		 e.getPoint().x + 1 - topLeft.x - 1, 
		 e.getPoint().y + 1 - topLeft.y - 1);

      // Store the location of this drag event for the next
      // time around.
      lastDrag = e.getPoint();

      // Dispose of the graphics. 
      g.dispose();
    }
  }
}

